#!/usr/bin/python
import sys, time, smbus

# bus, address
bus = 1
addr = 0x77
# oversampling settings [0,1,2,3]
oss = 3
# altitude
altitude = 330.0
debug = 0

# some globales
AC1 = 0
AC2 = 0
AC3 = 0
AC4 = 0
AC5 = 0
AC6 = 0
B1 = 0
B2 = 0
MB = 0
MC = 0
MD = 0

def read_signed(reg):
  try:
    msb = i2c.read_byte_data(addr, reg)
    if (msb & 0x80):
      msb -= 256
    lsb = i2c.read_byte_data(addr, reg+1)
    if (debug):
      print "read: %02x %04x " % (reg, (msb<<8|lsb))
      # print "read: %d " % ((msb<<8|lsb))
    return (msb<<8|lsb)   
  except IOError, err:
    print "Error read register %02x." % reg

def read_unsigned(reg):
  try:
    msb = i2c.read_byte_data(addr, reg)
    lsb = i2c.read_byte_data(addr, reg+1)
    if (debug):
      print "read: %02x %04x " % (reg, (msb<<8|lsb))
      #print "read: %d " % ((msb<<8|lsb))
    return (msb<<8|lsb)   
  except IOError, err:
    print "Error read register %02x." % reg

def write_command(reg, cmd):
  try:
    i2c.write_byte_data(addr, reg, cmd)
    if (debug):
      print "write: %02x to %02x " % (cmd, reg)
  except IOError, err:
    print "Error write to register %02x." % reg

def read_temp():
  # start temp conversation
  write_command(0xF4, 0x2E)
  time.sleep(0.005)
  # read temp
  return (read_unsigned(0xF6))

def convert_temp(UT):
  # calculate temp
  X1 = ((UT - AC6) * AC5) >> 15
  X2 = (MC << 11) / (X1 + MD)
  B5 = X1 + X2
  T = ((B5 + 8) >> 4) / 10.0
  return (T)

def read_pressure(oss=0):
  # oss - oversampling rate 
  # start pressure conversation
  try:
    write_command(0xF4, 0x34 + (oss<<6))
    # wait for conversion
    if (oss == 0):
      time.sleep(0.005)    # 4.5ms
    elif (oss == 1):
      time.sleep(0.008)    # 7.5ms
    elif (oss == 2):
      time.sleep(0.014)    # 13.5ms
    else:
      time.sleep(0.026)    # 25.5ms
    # read raw pressure
    reg = 0xF6
    msb = i2c.read_byte_data(addr, reg)
    lsb = i2c.read_byte_data(addr, reg+1)
    xlsb = i2c.read_byte_data(addr, reg+2)
    raw_pressure = ((msb << 16) + (lsb << 8) + xlsb) >> (8 - oss)
    if (debug):
      print "read raw pressure: %02x %04x " % (reg, raw_pressure)
    return (raw_pressure)   
  except IOError, err:
    print "Error read register %02x." % reg

def convert_pressure(UP, UT, oss):
  # calc temp again
  X1 = ((UT - AC6) * AC5) >> 15
  X2 = (MC << 11) / (X1 + MD)
  B5 = X1 + X2
  # calculate pressure
  B6 = B5 - 4000
  X1 = (B2 * (B6 * B6) >> 12 ) >> 11
  X2 = (AC2 * B6) >> 11
  X3 = X1 + X2
  B3 = (((AC1 * 4 + X3) << oss) + 2) / 4
  X1 = (AC3 * B6) >> 13
  X2 = (B1 * ((B6 * B6) >> 12 )) >> 16
  X3 = ((X1 + X2) + 2) >> 2
  B4 = (AC4 * (X3 + 32768)) >> 15
  B7 = (UP - B3) * (50000 >> oss)
  if (B7 < 0x80000000):
    p = (B7 * 2) / B4
  else:
    p = (B7 / B4) * 2
  X1 = (p >> 8) * (p >> 8)
  X1 = (X1 * 3038) >> 16
  X2 = (-7357 * p) >> 16
  p = p + ((X1 + X2 + 3791) >> 4)
  return (p)

def read_calibration_data():
  global AC1
  AC1 = read_signed(0xAA)
  global AC2
  AC2 = read_signed(0xAC)
  global AC3
  AC3 = read_signed(0xAE)
  global AC4
  AC4 = read_unsigned(0xB0)
  global AC5
  AC5 = read_unsigned(0xB2)
  global AC6
  AC6 = read_unsigned(0xB4)
  global B1
  B1 = read_signed(0xB6)
  global B2
  B2 = read_signed(0xB8)
  global MB
  MB = read_signed(0xBA)
  global MC
  MC = read_signed(0xBC)
  global MD
  MD = read_signed(0xBE)

# init i2c device
i2c = smbus.SMBus(bus)

read_calibration_data()

while True:
  # enter main loop
  UT = read_temp()
  T = convert_temp(UT)
  print "Temperatur:              %.1f C" % T

  UP = read_pressure(oss)
  P = float(convert_pressure(UP, UT, oss))
  print "Pressure (absolute):    %0.1f hPa" % (P/100)

  # pressure at sea level
  p1 = float(P)
  po = p1 / pow(1-(altitude/44330), 5.255)
  print "Pressure at sea level: %0.1f hPa" % (po/100)
  print

  time.sleep(5)



